//---------------------------------------------------------------------
//  Pass 0 - Environment
//---------------------------------------------------------------------
float4 vViewPosition;
float4x4 matViewProjection;

struct VS_INPUT_0 
{
   float4 Position : POSITION0;
   //float3 Normal   : NORMAL0;
   
};

struct VS_OUTPUT_0 
{
   float4 Position : POSITION0;
   float3 Texcoord : TEXCOORD0;
   
};

VS_OUTPUT_0 vs_main_0( VS_INPUT_0 Input )
{
   VS_OUTPUT_0 Output;

   Output.Position = mul( float4(Input.Position.xyz + vViewPosition.xyz, 1), matViewProjection );
   Output.Texcoord = normalize(Input.Position.xyz);
   
   return( Output );
   
}

texture EnvironmentMap;
sampler Environment = sampler_state
{
    Texture = <EnvironmentMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};

struct PS_INPUT_0 
{
   float3 Texcoord : TEXCOORD0;
   
};

float4 ps_main_0(PS_INPUT_0 Input) : COLOR0
{   
   return( texCUBE(Environment, Input.Texcoord) );
   
}

//---------------------------------------------------------------------
//  Pass 1 : Reflection or Refraction
//---------------------------------------------------------------------
//float4 vViewPosition;
float fRef;
float4x4 matWorldView;
float4x4 matWorldViewProjection;
float4x4 matWorldViewInverseTranspose;

struct VS_INPUT_1 
{
   float4 Position : POSITION0;
   float3 Normal   : NORMAL0;
};

struct VS_OUTPUT_1 
{
   float4 Position : POSITION0;
   float3 Texcoord : TEXCOORD0;
};

VS_OUTPUT_1 vs_main_1( VS_INPUT_1 Input )
{
   VS_OUTPUT_1 Output;

   Output.Position = mul( Input.Position, matWorldViewProjection );

   float4 Position = mul(Input.Position, matWorldView);
   float3 Eye = normalize(vViewPosition - Input.Position);   // local or world space
   //float4 N = normalize( mul(Input.Normal, matWorldView) );
   //float3 Eye = normalize(-Position);   // View Space
   //float3 Reflection = normalize( 2 * dot(N, Eye) * N - Eye );
   
   // Reflection : 2.0, Refraction : 0.5
   float3 Reflection = normalize( fRef * dot(Input.Normal, Eye) * Input.Normal - Eye );
   //float3 Reflection = normalize( reflect(-Eye,Input.Normal) );
   
   Output.Texcoord = Reflection;
   
   return( Output );
   
}

//samplerCUBE Texture0;

struct PS_INPUT_1 
{
   float3 Texcoord : TEXCOORD0;
   
};

float4 ps_main_1(PS_INPUT_1 Input) : COLOR0
{   
   return( texCUBE(Environment, Input.Texcoord) );
   
}

//=================================================================================================================================
//Technique
//=================================================================================================================================
technique TShader
{
    pass P0
    {
        VertexShader = compile vs_2_0 vs_main_0();
        PixelShader  = compile ps_2_0 ps_main_0();
    }
    pass P1
    {
        VertexShader = compile vs_2_0 vs_main_1();
        PixelShader  = compile ps_2_0 ps_main_1();
    }
}